@@ -1,39 +1,40 @@
-##
 #
-# Copyright 2012 Ghent University
-# Copyright 2009-2012 Stijn De Weirdt
+# Copyright 2012-2020 Ghent University
 #
-# This file is part of VSC-tools,
+# This file is part of vsc-base,
 # originally created by the HPC team of Ghent University (http://ugent.be/hpc/en),
 # with support of Ghent University (http://ugent.be/hpc),
-# the Flemish Supercomputer Centre (VSC) (https://vscentrum.be/nl/en),
-# the Hercules foundation (http://www.herculesstichting.be/in_English)
+# the Flemish Supercomputer Centre (VSC) (https://www.vscentrum.be),
+# the Flemish Research Foundation (FWO) (http://www.fwo.be/en)
 # and the Department of Economy, Science and Innovation (EWI) (http://www.ewi-vlaanderen.be/en).
 #
-# http://github.com/hpcugent/VSC-tools
+# https://github.com/hpcugent/vsc-base
 #
-# VSC-tools is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation v2.
+# vsc-base is free software: you can redistribute it and/or modify
+# it under the terms of the GNU Library General Public License as
+# published by the Free Software Foundation, either version 2 of
+# the License, or (at your option) any later version.
 #
-# VSC-tools is distributed in the hope that it will be useful,
+# vsc-base is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
+# GNU Library General Public License for more details.
+#
+# You should have received a copy of the GNU Library General Public License
+# along with vsc-base. If not, see <http://www.gnu.org/licenses/>.
 #
-# You should have received a copy of the GNU General Public License
-# along with VSC-tools. If not, see <http://www.gnu.org/licenses/>.
-##
 """
-module with various convenience functions and classes to deal with date, time and timezone
+Module with various convenience functions and classes to deal with date, time and timezone
+
+@author: Stijn De Weirdt (Ghent University)
 """
 
 import calendar
-import operator
 import re
 import time as _time
 from datetime import tzinfo, timedelta, datetime, date
 
+
 class FancyMonth:
     """Convenience class for month math"""
     def __init__(self, tmpdate=None, year=None, month=None, day=None):
@@ -51,7 +52,6 @@ class FancyMonth:
 
         self.date = date(year, month, day)
 
-
         self.first = None
         self.last = None
         self.nrdays = None
@@ -65,7 +65,43 @@ class FancyMonth:
 
     def set_details(self):
         """Get first/last day of the month of date"""
-        c = calendar.Calendar()
+        class MyCalendar(object):
+            """Backport minimal calendar.Calendar code from 2.7 to support itermonthdays in 2.4"""
+            def __init__(self, firstweekday=0):
+                self.firstweekday = firstweekday  # 0 = Monday, 6 = Sunday
+
+            def itermonthdates(self, year, month):
+                """
+                Return an iterator for one month. The iterator will yield datetime.date
+                values and will always iterate through complete weeks, so it will yield
+                dates outside the specified month.
+                """
+                _date = date(year, month, 1)
+                # Go back to the beginning of the week
+                days = (_date.weekday() - self.firstweekday) % 7
+                _date -= timedelta(days=days)
+                oneday = timedelta(days=1)
+                while True:
+                    yield _date
+                    _date += oneday
+                    if _date.month != month and _date.weekday() == self.firstweekday:
+                        break
+
+            def itermonthdays(self, year, month):
+                """
+                Like itermonthdates(), but will yield day numbers. For days outside
+                the specified month the day number is 0.
+                """
+                for _date in self.itermonthdates(year, month):
+                    if _date.month != month:
+                        yield 0
+                    else:
+                        yield _date.day
+
+        if 'Calendar' in dir(calendar):  # py2.5+
+            c = calendar.Calendar()
+        else:
+            c = MyCalendar()
         self.nrdays = len([x for x in c.itermonthdays(self.date.year, self.date.month) if x > 0])
 
         self.first = date(self.date.year, self.date.month, 1)
@@ -86,7 +122,7 @@ class FancyMonth:
     def number(self, otherdate):
         """Calculate the number of months between this month (date actually) and otherdate
         """
-        if self.include == False:
+        if self.include is False:
             msg = "number: include=False not implemented"
             raise(Exception(msg))
         else:
@@ -99,22 +135,22 @@ class FancyMonth:
 
         return nr
 
-    def get_other(self, shift= -1):
+    def get_other(self, shift=-1):
         """Return month that is shifted shift months: negative integer is in past, positive is in future"""
         new = self.date.year * 12 + self.date.month - 1 + shift
-        return self.__class__(date(new // 12, new % 12 + 1, 01))
+        return self.__class__(date(new // 12, new % 12 + 1, 1))
 
     def interval(self, otherdate):
         """Return time ordered list of months between date and otherdate"""
-        if self.include == False:
+        if self.include is False:
             msg = "interval: include=False not implemented"
             raise(Exception(msg))
         else:
             nr = self.number(otherdate)
-            startdate, enddate = self.get_start_end(otherdate)
+            startdate, _ = self.get_start_end(otherdate)
 
             start = self.__class__(startdate)
-            all_dates = [ start.get_other(m)  for m in range(nr)]
+            all_dates = [start.get_other(m) for m in range(nr)]
 
         return all_dates
 
@@ -154,10 +190,16 @@ class FancyMonth:
 
         return res
 
+
 def date_parser(txt):
-    """Parse txt: date YYYY-MM-DD in datetime.date
-        also (BEGIN|END)(THIS|LAST|NEXT)MONTH
-             (BEGIN | END)(JANUARY | FEBRUARY | MARCH | APRIL | MAY | JUNE | JULY | AUGUST | SEPTEMBER | OCTOBER | NOVEMBER | DECEMBER)
+    """Parse txt
+
+    @type txt: string
+
+    @param txt: date to be parsed. Usually in C{YYYY-MM-DD} format,
+    but also C{(BEGIN|END)(THIS|LAST|NEXT)MONTH}, or even
+    C{(BEGIN | END)(JANUARY | FEBRUARY | MARCH | APRIL | MAY | JUNE | JULY | AUGUST | SEPTEMBER | OCTOBER | NOVEMBER |
+    DECEMBER)}
     """
 
     reserveddate = ('TODAY',)
@@ -166,8 +208,10 @@ def date_parser(txt):
     if txt.endswith('MONTH'):
         m = FancyMonth()
         res = m.parser(txt)
-    elif reduce(operator.or_, testsupportedmonths):  # TODO replace with any()
-        m = FancyMonth(month=testsupportedmonths.index(True) + 1)
+    elif any(testsupportedmonths):
+        # set day=1 or this will fail on days with an index more then the count of days then the month you want to parse
+        # e.g. will fail on 31'st when trying to parse april
+        m = FancyMonth(month=testsupportedmonths.index(True) + 1, day=1)
         res = m.parser(txt)
     elif txt in reserveddate:
         if txt in ('TODAY',):
@@ -187,6 +231,7 @@ def date_parser(txt):
 
     return res
 
+
 def datetime_parser(txt):
     """Parse txt: tmpdate YYYY-MM-DD HH:MM:SS.mmmmmm in datetime.datetime
         - date part is parsed with date_parser
@@ -196,23 +241,28 @@ def datetime_parser(txt):
 
     datetuple = [tmpdate.year, tmpdate.month, tmpdate.day]
     if len(tmpts) > 1:
-        ## add hour and minutes
+        # add hour and minutes
         datetuple.extend([int(x) for x in tmpts[1].split(':')[:2]])
 
         try:
             sects = tmpts[1].split(':')[2].split('.')
-        except:
+        except (AttributeError, IndexError):
             sects = [0]
-        ## add seconds
+        # add seconds
         datetuple.append(int(sects[0]))
         if len(sects) > 1:
-            ## add microseconds
+            # add microseconds
             datetuple.append(int(float('.%s' % sects[1]) * 10 ** 6))
 
     res = datetime(*datetuple)
 
     return res
 
+
+def timestamp_parser(timestamp):
+    """Parse timestamp to datetime"""
+    return datetime.fromtimestamp(float(timestamp))
+
 #
 # example code from http://docs.python.org/library/datetime.html
 # Implements Local, the local timezone
@@ -221,52 +271,57 @@ def datetime_parser(txt):
 ZERO = timedelta(0)
 HOUR = timedelta(hours=1)
 
-# A UTC class.
 
+# A UTC class.
 class UTC(tzinfo):
     """UTC"""
 
-    def utcoffset(self, dt):
+    def utcoffset(self, dt): # pylint:disable=unused-argument
         return ZERO
 
-    def tzname(self, dt):
+    def tzname(self, dt): # pylint:disable=unused-argument
         return "UTC"
 
-    def dst(self, dt):
+    def dst(self, dt): # pylint:disable=unused-argument
         return ZERO
 
 utc = UTC()
 
-# A class building tzinfo objects for fixed-offset time zones.
-# Note that FixedOffset(0, "UTC") is a different way to build a
-# UTC tzinfo object.
 
 class FixedOffset(tzinfo):
-    """Fixed offset in minutes east from UTC."""
+    """Fixed offset in minutes east from UTC.
+
+    This is a class for building tzinfo objects for fixed-offset time zones.
+    Note that FixedOffset(0, "UTC") is a different way to build a
+    UTC tzinfo object.
+    """
     def __init__(self, offset, name):
         self.__offset = timedelta(minutes=offset)
         self.__name = name
 
-    def utcoffset(self, dt):
+    def utcoffset(self, dt): # pylint:disable=unused-argument
         return self.__offset
 
-    def tzname(self, dt):
+    def tzname(self, dt): # pylint:disable=unused-argument
         return self.__name
 
-    def dst(self, dt):
+    def dst(self, dt): # pylint:disable=unused-argument
         return ZERO
 
-# A class capturing the platform's idea of local time.
 
-STDOFFSET = timedelta(seconds= -_time.timezone)
+STDOFFSET = timedelta(seconds=-_time.timezone)
 if _time.daylight:
-    DSTOFFSET = timedelta(seconds= -_time.altzone)
+    DSTOFFSET = timedelta(seconds=-_time.altzone)
 else:
     DSTOFFSET = STDOFFSET
 
 DSTDIFF = DSTOFFSET - STDOFFSET
 
+
 class LocalTimezone(tzinfo):
+    """
+    A class capturing the platform's idea of local time.
+    """
 
     def utcoffset(self, dt):
         if self._isdst(dt):
@@ -292,6 +347,3 @@ class LocalTimezone(tzinfo):
         return tt.tm_isdst > 0
 
 Local = LocalTimezone()
-
-
-
